@@ -279,7 +279,10 @@ class ConfigCollection:
 
     @classmethod
     def from_github_repo(
-        cls, repo_url: Optional[str] = None, is_private: bool = False
+        cls,
+        repo_url: Optional[str] = None,
+        is_private: bool = False,
+        path: Optional[str] = None,
     ) -> "ConfigCollection":
         """Pull in external config files."""
         # download files to tmp directory
@@ -288,10 +291,19 @@ class ConfigCollection:
                 repo = Repo(repo_url)
                 tmp_dir = Path(repo_url)
             else:
-                repo = Repo.clone_from(repo_url or cls.repo_url, tmp_dir)
+                if repo_url is not None and "/tree/" in repo_url:
+                    repo_url, tree = repo_url.split("/tree/")
+                    branch, path = tree.split("/", 1)
+                    repo = Repo.clone_from(repo_url or cls.repo_url, tmp_dir)
+                    repo.git.checkout(branch)
+                else:
+                    repo = Repo.clone_from(repo_url or cls.repo_url, tmp_dir)
 
             external_configs = []
 
+            if path is not None:
+                tmp_dir = tmp_dir / path
+
             for config_file in tmp_dir.glob("*.toml"):
                 last_modified = next(repo.iter_commits("HEAD", paths=config_file)).committed_date
                 config_json = toml.load(config_file)
@@ -313,7 +325,7 @@ class ConfigCollection:
                 )
 
             outcomes = []
-            for outcome_file in tmp_dir.glob(f"**/{OUTCOMES_DIR}/*/*.toml"):
+            for outcome_file in tmp_dir.glob(f"{OUTCOMES_DIR}/*/*.toml"):
                 commit_hash = next(repo.iter_commits("HEAD", paths=outcome_file)).hexsha
 
                 outcomes.append(
@@ -327,14 +339,14 @@ class ConfigCollection:
                 )
 
             default_configs = []
-            for default_config_file in tmp_dir.glob(f"**/{DEFAULTS_DIR}/*.toml"):
+            for default_config_file in tmp_dir.glob(f"{DEFAULTS_DIR}/*.toml"):
                 last_modified = next(
                     repo.iter_commits("HEAD", paths=default_config_file)
                 ).committed_date
 
                 default_config_json = toml.load(default_config_file)
 
-                if "project" in config_json:
+                if "project" in default_config_json:
                     # opmon spec
                     spec = MonitoringSpec.from_dict(default_config_json)
                 else:
@@ -351,7 +363,7 @@ class ConfigCollection:
                 )
 
             definitions = []
-            for definitions_config_file in tmp_dir.glob(f"**/{DEFINITIONS_DIR}/*.toml"):
+            for definitions_config_file in tmp_dir.glob(f"{DEFINITIONS_DIR}/*.toml"):
                 last_modified = next(
                     repo.iter_commits("HEAD", paths=definitions_config_file)
                 ).committed_date
@@ -367,7 +379,7 @@ class ConfigCollection:
                 )
 
             functions_spec = None
-            for functions_file in tmp_dir.glob(f"**/{DEFINITIONS_DIR}/{FUNCTIONS_FILE}"):
+            for functions_file in tmp_dir.glob(f"{DEFINITIONS_DIR}/{FUNCTIONS_FILE}"):
                 functions_spec = FunctionsSpec.from_dict(toml.load(functions_file))
 
         return cls(external_configs, outcomes, default_configs, definitions, functions_spec)
@@ -520,11 +532,13 @@ class ConfigCollection:
         other_definitions = {
             definition.slug: definition for definition in deepcopy(other.definitions)
         }
-        definitions = {definition.slug: definition for definition in deepcopy(self.definitions)}
+        definitions = {
+            definition.slug: deepcopy(definition) for definition in deepcopy(self.definitions)
+        }
 
         for slug, definition in other_definitions.items():
             if slug not in definitions:
-                definitions[slug] = definition
+                definitions[slug] = deepcopy(definition)
             else:
                 definitions[slug].spec.merge(definition.spec)
 
@@ -536,7 +550,7 @@ class ConfigCollection:
 
         for slug, default in other_defaults.items():
             if slug not in defaults:
-                defaults[slug] = default
+                defaults[slug] = deepcopy(default)
             else:
                 defaults[slug].spec.merge(default.spec)
 
